@@ -1,7 +1,7 @@
 /* -*- c++ -*- */
 /*
  * Gqrx SDR: Software defined radio receiver powered by GNU Radio and Qt
- *           http://gqrx.dk/
+ *           https://gqrx.dk/
  *
  * Copyright 2011-2014 Alexandru Csete OZ9AEC.
  * Copyright (C) 2013 by Elias Oenal <EliasOenal@gmail.com>
@@ -38,6 +38,7 @@
 #include <QMessageBox>
 #include <QPushButton>
 #include <QResource>
+#include <QShortcut>
 #include <QString>
 #include <QTextBrowser>
 #include <QTextCursor>
@@ -48,6 +49,8 @@
 #include <QSvgWidget>
 #include "qtgui/ioconfig.h"
 #include "mainwindow.h"
+#include "qtgui/dxc_options.h"
+#include "qtgui/dxc_spots.h"
 
 /* Qt Designer files */
 #include "ui_mainwindow.h"
@@ -57,6 +60,7 @@
 #include "remote_control_settings.h"
 
 #include "qtgui/bookmarkstaglist.h"
+#include "qtgui/bandplan.h"
 
 MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     QMainWindow(parent),
@@ -69,7 +73,9 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     dec_afsk1200(0)
 {
     ui->setupUi(this);
+    BandPlan::create();
     Bookmarks::create();
+    DXCSpots::create();
 
     /* Initialise default configuration directory */
     QByteArray xdg_dir = qgetenv("XDG_CONFIG_HOME");
@@ -122,13 +128,18 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     // create I/Q tool widget
     iq_tool = new CIqTool(this);
 
+    // create DXC Objects
+    dxc_options = new DXCOptions(this);
+
     /* create dock widgets */
     uiDockRxOpt = new DockRxOpt();
     uiDockRDS = new DockRDS();
     uiDockAudio = new DockAudio();
     uiDockInputCtl = new DockInputCtl();
     uiDockFft = new DockFft();
+    BandPlan::Get().setConfigDir(m_cfg_dir);
     Bookmarks::Get().setConfigDir(m_cfg_dir);
+    BandPlan::Get().load();
     uiDockBookmarks = new DockBookmarks(this);
 
     // setup some toggle view shortcuts
@@ -139,6 +150,10 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     uiDockBookmarks->toggleViewAction()->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_B));
     ui->mainToolBar->toggleViewAction()->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_T));
 
+    /* frequency setting shortcut */
+    QShortcut *freq_shortcut = new QShortcut(QKeySequence(Qt::Key_F), this);
+    QObject::connect(freq_shortcut, &QShortcut::activated, this, &MainWindow::frequencyFocusShortcut);
+
     setCorner(Qt::TopLeftCorner, Qt::LeftDockWidgetArea);
     setCorner(Qt::TopRightCorner, Qt::RightDockWidgetArea);
     setCorner(Qt::BottomLeftCorner, Qt::BottomDockWidgetArea);
@@ -208,6 +223,8 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(uiDockRxOpt, SIGNAL(fmEmphSelected(double)), this, SLOT(setFmEmph(double)));
     connect(uiDockRxOpt, SIGNAL(amDcrToggled(bool)), this, SLOT(setAmDcr(bool)));
     connect(uiDockRxOpt, SIGNAL(cwOffsetChanged(int)), this, SLOT(setCwOffset(int)));
+    connect(uiDockRxOpt, SIGNAL(amSyncDcrToggled(bool)), this, SLOT(setAmSyncDcr(bool)));
+    connect(uiDockRxOpt, SIGNAL(amSyncPllBwSelected(float)), this, SLOT(setAmSyncPllBw(float)));
     connect(uiDockRxOpt, SIGNAL(agcToggled(bool)), this, SLOT(setAgcOn(bool)));
     connect(uiDockRxOpt, SIGNAL(agcHangToggled(bool)), this, SLOT(setAgcHang(bool)));
     connect(uiDockRxOpt, SIGNAL(agcThresholdChanged(int)), this, SLOT(setAgcThreshold(int)));
@@ -237,6 +254,7 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(uiDockFft, SIGNAL(resetFftZoom()), ui->plotter, SLOT(resetHorizontalZoom()));
     connect(uiDockFft, SIGNAL(gotoFftCenter()), ui->plotter, SLOT(moveToCenterFreq()));
     connect(uiDockFft, SIGNAL(gotoDemodFreq()), ui->plotter, SLOT(moveToDemodFreq()));
+    connect(uiDockFft, SIGNAL(bandPlanChanged(bool)), ui->plotter, SLOT(toggleBandPlan(bool)));
     connect(uiDockFft, SIGNAL(wfColormapChanged(const QString)), ui->plotter, SLOT(setWfColormap(const QString)));
     connect(uiDockFft, SIGNAL(wfColormapChanged(const QString)), uiDockAudio, SLOT(setWfColormap(const QString)));
 
@@ -260,6 +278,8 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(uiDockBookmarks, SIGNAL(newBookmarkActivated(qint64, QString, int)), this, SLOT(onBookmarkActivated(qint64, QString, int)));
     connect(uiDockBookmarks->actionAddBookmark, SIGNAL(triggered()), this, SLOT(on_actionAddBookmark_triggered()));
 
+    //DXC Spots
+    connect(&DXCSpots::Get(), SIGNAL(dxcSpotsUpdated()), this, SLOT(updateClusterSpots()));
 
     // I/Q playback
     connect(iq_tool, SIGNAL(startRecording(QString)), this, SLOT(startIqRecording(QString)));
@@ -284,12 +304,13 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     connect(ui->plotter, SIGNAL(newFilterFreq(int, int)), remote, SLOT(setPassband(int, int)));
     connect(remote, SIGNAL(newPassband(int)), this, SLOT(setPassband(int)));
     connect(remote, SIGNAL(gainChanged(QString, double)), uiDockInputCtl, SLOT(setGain(QString,double)));
+    connect(remote, SIGNAL(dspChanged(bool)), this, SLOT(on_actionDSP_triggered(bool)));
 
     rds_timer = new QTimer(this);
     connect(rds_timer, SIGNAL(timeout()), this, SLOT(rdsTimeout()));
 
     // enable frequency tooltips on FFT plot
-#ifdef Q_OS_MAC
+#ifdef Q_OS_DARWIN
     ui->plotter->setTooltipsEnabled(false);
 #else
     ui->plotter->setTooltipsEnabled(true);
@@ -299,6 +320,9 @@ MainWindow::MainWindow(const QString cfgfile, bool edit_conf, QWidget *parent) :
     // restored because device probing might change the device configuration
     CIoConfig::getDeviceList(devList);
 
+    m_recent_config = new RecentConfig(m_cfg_dir, ui->menu_RecentConfig);
+    connect(m_recent_config, SIGNAL(loadConfig(const QString &)), this, SLOT(loadConfigSlot(const QString &)));
+
     // restore last session
     if (!loadConfig(cfgfile, true, true))
     {
@@ -370,7 +394,10 @@ MainWindow::~MainWindow()
         delete m_settings;
     }
 
+    delete m_recent_config;
+
     delete iq_tool;
+    delete dxc_options;
     delete ui;
     delete uiDockRxOpt;
     delete uiDockAudio;
@@ -415,7 +442,12 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash,
     qDebug() << "Loading configuration from:" << cfgfile;
 
     if (m_settings)
+    {
+        // set current config to not crashed before loading new config
+        m_settings->setValue("crashed", false);
+        m_settings->sync();
         delete m_settings;
+    }
 
     if (QDir::isAbsolutePath(cfgfile))
         m_settings = new QSettings(cfgfile, QSettings::IniFormat);
@@ -517,7 +549,17 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash,
     }
 
     QString outdev = m_settings->value("output/device", "").toString();
-    rx->set_output_device(outdev.toStdString());
+
+    try {
+        rx->set_output_device(outdev.toStdString());
+    } catch (std::exception &x) {
+        QMessageBox::warning(nullptr,
+                         QObject::tr("Failed to set output device"),
+                         QObject::tr("<p><b>%1</b></p>"
+                                     "Please select another device.")
+                                 .arg(x.what()),
+                         QMessageBox::Ok);
+    }
 
     int_val = m_settings->value("input/sample_rate", 0).toInt(&conv_ok);
     if (conv_ok && (int_val > 0))
@@ -592,6 +634,7 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash,
     uiDockRxOpt->readSettings(m_settings);
     uiDockFft->readSettings(m_settings);
     uiDockAudio->readSettings(m_settings);
+    dxc_options->readSettings(m_settings);
 
     {
         int64_val = m_settings->value("input/frequency", 14236000).toLongLong(&conv_ok);
@@ -632,6 +675,8 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash,
        ui->actionRemoteControl->setChecked(true);
     }
 
+    emit m_recent_config->configLoaded(m_settings->fileName());
+
     return conf_ok;
 }
 
@@ -644,7 +689,7 @@ bool MainWindow::loadConfig(const QString cfgfile, bool check_crash,
  * assumed to be the name of a file under m_cfg_dir.
  *
  * If cfgfile already exists it will be overwritten (we assume that a file
- * selection dialog has already asked for confirmation of overwrite.
+ * selection dialog has already asked for confirmation of overwrite).
  *
  * Since QSettings does not support "save as" we do this by copying the current
  * settings to a new file.
@@ -665,6 +710,7 @@ bool MainWindow::saveConfig(const QString cfgfile)
 
     if (newfile == oldfile) {
         qDebug() << "New file is equal to old file => SYNCING...";
+        emit m_recent_config->configSaved(newfile);
         return true;
     }
 
@@ -678,10 +724,6 @@ bool MainWindow::saveConfig(const QString cfgfile)
     }
     if (QFile::copy(oldfile, newfile))
     {
-        // ensure that old config has crash cleared
-        m_settings->setValue("crashed", false);
-        m_settings->sync();
-
         loadConfig(cfgfile, false, false);
         return true;
     }
@@ -711,6 +753,7 @@ void MainWindow::storeSession()
 
         remote->saveSettings(m_settings);
         iq_tool->saveSettings(m_settings);
+        dxc_options->saveSettings(m_settings);
 
         {
             int     flo, fhi;
@@ -1066,6 +1109,13 @@ void MainWindow::selectDemod(int mode_idx)
         click_res = 100;
         break;
 
+    case DockRxOpt::MODE_AM_SYNC:
+        rx->set_demod(receiver::RX_DEMOD_AMSYNC);
+        ui->plotter->setDemodRanges(-40000, -200, 200, 40000, true);
+        uiDockAudio->setFftRange(0,6000);
+        click_res = 100;
+        break;
+
     case DockRxOpt::MODE_NFM:
         ui->plotter->setDemodRanges(-40000, -1000, 1000, 40000, true);
         uiDockAudio->setFftRange(0, 5000);
@@ -1193,6 +1243,27 @@ void MainWindow::setCwOffset(int offset)
     rx->set_cw_offset(offset);
 }
 
+/**
+ * @brief AM-Sync DCR status changed (slot).
+ * @param enabled Whether DCR is enabled or not.
+ */
+void MainWindow::setAmSyncDcr(bool enabled)
+{
+    rx->set_amsync_dcr(enabled);
+}
+
+/**
+ * @brief New AM-Sync PLL BW selected.
+ * @param pll_bw The new PLL BW.
+ */
+void MainWindow::setAmSyncPllBw(float pll_bw)
+{
+    qDebug() << "AM-Sync PLL BW: " << pll_bw;
+
+    /* receiver will check range */
+    rx->set_amsync_pll_bw(pll_bw);
+}
+
 /**
  * @brief Audio gain changed.
  * @param value The new audio gain in dB.
@@ -1295,7 +1366,6 @@ void MainWindow::iqFftTimeout()
     unsigned int    fftsize;
     unsigned int    i;
     float           pwr_scale;
-    std::complex<float> pt;     /* a single FFT point used in calculations */
 
     // FIXME: fftsize is a reference
     rx->get_iq_fft_data(d_fftData, fftsize);
@@ -1715,31 +1785,13 @@ void MainWindow::setPeakDetection(bool enabled)
     ui->plotter->setPeakDetection(enabled ,2);
 }
 
-/**
- * @brief Force receiver reconfiguration.
- *
- * Aka. jerky dongle workaround.
- *
- * This function forces a receiver reconfiguration by sending a fake
- * selectDemod() signal using the current demodulator selection.
- *
- * This function provides a workaround for the "jerky streaming" that has
- * been experienced using some RTL-SDR dongles when DSP processing is
- * started. The jerkyness disappears when the receiver is reconfigured
- * by selecting a new demodulator.
- */
-/*void MainWindow::forceRxReconf()
-{
-    qDebug() << "Force RX reconf (jerky dongle workarond)...";
-    selectDemod(uiDockRxOpt->currentDemod());
-}*/
-
 /**
  * @brief Start/Stop DSP processing.
  * @param checked Flag indicating whether DSP processing should be ON or OFF.
  *
  * This slot is executed when the actionDSP is toggled by the user. This can
- * either be via the menu bar or the "power on" button in the main toolbar.
+ * either be via the menu bar or the "power on" button in the main toolbar or
+ * by remote control.
  */
 void MainWindow::on_actionDSP_triggered(bool checked)
 {
@@ -1769,9 +1821,6 @@ void MainWindow::on_actionDSP_triggered(bool checked)
         /* update menu text and button tooltip */
         ui->actionDSP->setToolTip(tr("Stop DSP processing"));
         ui->actionDSP->setText(tr("Stop DSP"));
-
-        // reconfigure RX after 1s to counteract possible jerky streaming from rtl dongles
-        //QTimer::singleShot(1000, this, SLOT(forceRxReconf()));
     }
     else
     {
@@ -1790,6 +1839,9 @@ void MainWindow::on_actionDSP_triggered(bool checked)
 
         ui->plotter->setRunningState(false);
     }
+
+    ui->actionDSP->setChecked(checked); //for remote control
+
 }
 
 /**
@@ -2035,6 +2087,7 @@ void MainWindow::on_actionAFSK1200_triggered()
         {
             dec_afsk1200 = new Afsk1200Win(this);
             connect(dec_afsk1200, SIGNAL(windowClosed()), this, SLOT(afsk1200win_closed()));
+            dec_afsk1200->setAttribute(Qt::WA_DeleteOnClose);
             dec_afsk1200->show();
 
             dec_timer->start(100);
@@ -2061,11 +2114,14 @@ void MainWindow::afsk1200win_closed()
     dec_timer->stop();
     rx->stop_sniffer();
 
-    /* delete decoder object */
-    delete dec_afsk1200;
     dec_afsk1200 = 0;
 }
 
+/** Show DXC Options. */
+void MainWindow::on_actionDX_Cluster_triggered()
+{
+    dxc_options->show();
+}
 
 /**
  * Cyclic processing for acquiring samples from receiver and processing them
@@ -2232,6 +2288,15 @@ void MainWindow::showSimpleTextFile(const QString &resource_path,
     // browser and layout deleted automatically
 }
 
+/**
+ * @brief Slot for handling loadConfig signals
+ * @param cfgfile
+ */
+void MainWindow::loadConfigSlot(const QString &cfgfile)
+{
+    loadConfig(cfgfile, cfgfile != m_settings->fileName(), cfgfile != m_settings->fileName());
+}
+
 /**
  * @brief Action: About gqrx.
  *
@@ -2250,7 +2315,7 @@ void MainWindow::on_actionAbout_triggered()
            "Funcube Dongle, RTL-SDR, Airspy, HackRF, RFSpace, BladeRF and USRP receivers."
            "</p>"
            "<p>You can download the latest version from the "
-           "<a href='http://gqrx.dk/'>Gqrx website</a>."
+           "<a href='https://gqrx.dk/'>Gqrx website</a>."
            "</p>"
            "<p>"
            "Gqrx is licensed under the <a href='http://www.gnu.org/licenses/gpl.html'>GNU General Public License</a>."
@@ -2345,3 +2410,13 @@ void MainWindow::on_actionAddBookmark_triggered()
         ui->plotter->updateOverlay();
     }
 }
+
+void MainWindow::updateClusterSpots()
+{
+    ui->plotter->updateOverlay();
+}
+
+void MainWindow::frequencyFocusShortcut()
+{
+    ui->freqCtrl->setFrequencyFocus();
+}
