@@ -1,7 +1,7 @@
 /* -*- c++ -*- */
 /*
  * Gqrx SDR: Software defined radio receiver powered by GNU Radio and Qt
- *           http://gqrx.dk/
+ *           https://gqrx.dk/
  *
  * Copyright 2011-2014 Alexandru Csete OZ9AEC.
  *
@@ -21,11 +21,6 @@
  * Boston, MA 02110-1301, USA.
  */
 #include <cmath>
-#include <iostream>
-#ifndef _MSC_VER
-#include <unistd.h>
-#endif
-
 #include <iostream>
 
 #include <gnuradio/prefs.h>
@@ -35,7 +30,6 @@
 
 #include "applications/gqrx/receiver.h"
 #include "dsp/correct_iq_cc.h"
-//#include "dsp/hbf_decim.h"
 #include "dsp/filter/fir_decim.h"
 #include "dsp/rx_fft.h"
 #include "receivers/nbrx.h"
@@ -182,28 +176,21 @@ void receiver::stop()
 
 /**
  * @brief Select new input device.
- *
- * @bug When using ALSA, program will crash if the new device
- *      is the same as the previously used device:
- *      audio_alsa_source[hw:1]: Device or resource busy
+ * @param device
  */
 void receiver::set_input_device(const std::string device)
 {
+#ifndef QT_NO_DEBUG_OUTPUT
+    std::cout << "Set input device:" << std::endl
+              << "  old: " << input_devstr << std::endl
+              << "  new: " << device << std::endl;
+#endif
+
     std::string error = "";
 
     if (device.empty())
         return;
 
-    if (input_devstr.compare(device) == 0)
-    {
-#ifndef QT_NO_DEBUG_OUTPUT
-        std::cout << "No change in input device:" << std::endl
-                  << "  old: " << input_devstr << std::endl
-                  << "  new: " << device << std::endl;
-#endif
-        return;
-    }
-
     input_devstr = device;
 
     // tb->lock() can hang occasionally
@@ -258,21 +245,14 @@ void receiver::set_input_device(const std::string device)
 }
 
 
-/** Select new audio output device. */
+/**
+ * @brief Select new audio output device.
+ * @param device
+ */
 void receiver::set_output_device(const std::string device)
 {
-    if (output_devstr.compare(device) == 0)
-    {
 #ifndef QT_NO_DEBUG_OUTPUT
-        std::cout << "No change in output device:" << std::endl
-                  << "  old: " << output_devstr << std::endl
-                  << "  new: " << device << std::endl;
-#endif
-        return;
-    }
-
-#ifndef QT_NO_DEBUG_OUTPUT
-    std::cout << "New audio output device:" << std::endl
+    std::cout << "Set output device:" << std::endl
               << "   old: " << output_devstr << std::endl
               << "   new: " << device << std::endl;
 #endif
@@ -288,21 +268,28 @@ void receiver::set_output_device(const std::string device)
     }
     audio_snk.reset();
 
+    try {
 #ifdef WITH_PULSEAUDIO
-    audio_snk = make_pa_sink(device, d_audio_rate, "GQRX", "Audio output");
+        audio_snk = make_pa_sink(device, d_audio_rate, "GQRX", "Audio output");
 #elif WITH_PORTAUDIO
-    audio_snk = make_portaudio_sink(device, d_audio_rate, "GQRX", "Audio output");
+        audio_snk = make_portaudio_sink(device, d_audio_rate, "GQRX", "Audio output");
 #else
-    audio_snk = gr::audio::sink::make(d_audio_rate, device, true);
+        audio_snk = gr::audio::sink::make(d_audio_rate, device, true);
 #endif
 
-    if (d_demod != RX_DEMOD_OFF)
-    {
-        tb->connect(audio_gain0, 0, audio_snk, 0);
-        tb->connect(audio_gain1, 0, audio_snk, 1);
-    }
+        if (d_demod != RX_DEMOD_OFF)
+        {
+            tb->connect(audio_gain0, 0, audio_snk, 0);
+            tb->connect(audio_gain1, 0, audio_snk, 1);
+        }
 
-    tb->unlock();
+        tb->unlock();
+
+    } catch (std::exception &x) {
+        tb->unlock();
+        // handle problems on non-freeing devices
+        throw x;
+    }
 }
 
 /** Get a list of available antenna connectors. */
@@ -863,11 +850,8 @@ receiver::status receiver::set_demod(rx_demod demod)
 {
     status ret = STATUS_OK;
 
-    // Allow reconf using same demod to provide a workaround
-    // for the "jerky streaming" we may experience with rtl
-    // dongles (the jerkyness disappears when we run this function)
-    //if (demod == d_demod)
-    //    return ret;
+    if (demod == d_demod)
+        return ret;
 
     // tb->lock() seems to hang occasioanlly
     if (d_running)
@@ -894,6 +878,11 @@ receiver::status receiver::set_demod(rx_demod demod)
         rx->set_demod(nbrx::NBRX_DEMOD_AM);
         break;
 
+    case RX_DEMOD_AMSYNC:
+        connect_all(RX_CHAIN_NBRX);
+        rx->set_demod(nbrx::NBRX_DEMOD_AMSYNC);
+        break;
+
     case RX_DEMOD_NFM:
         connect_all(RX_CHAIN_NBRX);
         rx->set_demod(nbrx::NBRX_DEMOD_FM);
@@ -960,6 +949,22 @@ receiver::status receiver::set_am_dcr(bool enabled)
     return STATUS_OK;
 }
 
+receiver::status receiver::set_amsync_dcr(bool enabled)
+{
+    if (rx->has_amsync())
+        rx->set_amsync_dcr(enabled);
+
+    return STATUS_OK;
+}
+
+receiver::status receiver::set_amsync_pll_bw(float pll_bw)
+{
+    if (rx->has_amsync())
+        rx->set_amsync_pll_bw(pll_bw);
+
+    return STATUS_OK;
+}
+
 receiver::status receiver::set_af_gain(float gain_db)
 {
     float k;
