@@ -54,6 +54,7 @@ CFreqCtrl::CFreqCtrl(QWidget *parent) :
     setFocusPolicy(Qt::StrongFocus);
     setMouseTracking(true);
     m_BkColor = QColor(0x1F, 0x1D, 0x1D, 0xFF);
+    m_InactiveColor = QColor(0x43, 0x43, 0x43, 0xFF);
     m_DigitColor = QColor(0xFF, 0xFF, 0xFF, 0xFF);
     m_HighlightColor = QColor(0x5A, 0x5A, 0x5A, 0xFF);
     m_UnitsColor = Qt::gray;
@@ -64,6 +65,7 @@ CFreqCtrl::CFreqCtrl(QWidget *parent) :
     m_LRMouseFreqSel = false;
     m_ActiveEditDigit = -1;
     m_ResetLowerDigits = true;
+    m_InvertScrolling = false;
     m_UnitsFont = QFont("Arial", 12, QFont::Normal);
     m_DigitFont = QFont("Arial", 12, QFont::Normal);
 
@@ -262,9 +264,11 @@ void CFreqCtrl::setDigitColor(QColor col)
 
 void CFreqCtrl::setUnit(FctlUnit unit)
 {
+    m_NumDigitsForUnit = 2;
     switch (unit)
     {
     case FCTL_UNIT_NONE:
+        m_NumDigitsForUnit = 0;
         m_DecPos = 0;
         m_UnitString = QString();
         break;
@@ -461,7 +465,8 @@ void CFreqCtrl::mousePressEvent(QMouseEvent *event)
 void CFreqCtrl::wheelEvent(QWheelEvent *event)
 {
     QPoint    pt = event->pos();
-    int       numDegrees = event->delta() / 8;
+    int       delta = m_InvertScrolling ? -event->angleDelta().y() : event->angleDelta().y();
+    int       numDegrees = delta / 8;
     int       numSteps = numDegrees / 15;
 
     for (int i = m_DigStart; i < m_NumDigits; i++)
@@ -510,6 +515,7 @@ void CFreqCtrl::keyPressEvent(QKeyEvent *event)
         moveCursorRight();
         fSkipMsg = true;
         break;
+    case Qt::Key_Backspace:
     case Qt::Key_Left:
         if (m_ActiveEditDigit != -1)
         {
@@ -559,8 +565,8 @@ void CFreqCtrl::drawBkGround(QPainter &Painter)
 
     // qDebug() <<rect;
     int    cellwidth = 100 * rect.width() /
-                       (100 * (m_NumDigits + 2) + (m_NumSeps * SEPRATIO_N) /
-                        SEPRATIO_D);
+                       (100 * (m_NumDigits + m_NumDigitsForUnit) +
+                        (m_NumSeps * SEPRATIO_N) / SEPRATIO_D);
     int    sepwidth = (SEPRATIO_N * cellwidth) / (100 * SEPRATIO_D);
     // qDebug() <<cellwidth <<sepwidth;
 
@@ -585,7 +591,7 @@ void CFreqCtrl::drawBkGround(QPainter &Painter)
     Painter.setPen(m_DigitColor);
 
     char    dgsep = ' ';       // digit group separator
-    int     digpos = rect.right() - 2 * cellwidth - 1; // starting digit x position
+    int     digpos = rect.right() - m_NumDigitsForUnit * cellwidth - 1; // starting digit x position
     for (int i = m_DigStart; i < m_NumDigits; i++)
     {
         if ((i > m_DigStart) && ((i % 3) == 0))
@@ -642,7 +648,7 @@ void CFreqCtrl::drawDigits(QPainter &Painter)
                 Painter.fillRect(m_DigitInfo[i].dQRect, m_BkColor);
 
             if (i >= m_LeadZeroPos)
-                Painter.setPen(m_BkColor);
+                Painter.setPen(m_InactiveColor);
             else
                 Painter.setPen(m_DigitColor);
 
@@ -845,3 +851,16 @@ void CFreqCtrl::cursorEnd()
                                     center()));
     }
 }
+
+void CFreqCtrl::setFrequencyFocus()
+{
+    uint8_t position = floor(log10(m_freq));
+    position = (uint8_t)fmax(position, 4);      // restrict min to 100s of kHz
+
+    QMouseEvent *mouseEvent = new QMouseEvent(QEvent::MouseMove,
+                                              m_DigitInfo[position].dQRect.center(),
+                                              Qt::NoButton,
+                                              Qt::NoButton,
+                                              Qt::NoModifier);
+    mouseMoveEvent(mouseEvent);
+}
