@@ -28,34 +28,6 @@
  * or implied, of Moe Wheatley.
  */
 #include <cmath>
-
-#ifndef _MSC_VER
-#include <sys/time.h>
-#else
-#include <Windows.h>
-#include <cstdint>
-
-int gettimeofday(struct timeval * tp, struct timezone * tzp)
-{
-    // Note: some broken versions only have 8 trailing zero's, the correct epoch has 9 trailing zero's
-    static const uint64_t EPOCH = ((uint64_t) 116444736000000000ULL);
-
-    SYSTEMTIME  system_time;
-    FILETIME    file_time;
-    uint64_t    time;
-
-    GetSystemTime( &system_time );
-    SystemTimeToFileTime( &system_time, &file_time );
-    time =  ((uint64_t)file_time.dwLowDateTime )      ;
-    time += ((uint64_t)file_time.dwHighDateTime) << 32;
-
-    tp->tv_sec  = (long) ((time - EPOCH) / 10000000L);
-    tp->tv_usec = (long) (system_time.wMilliseconds * 1000);
-    return 0;
-}
-
-#endif
-
 #include <math.h>
 #include <QColor>
 #include <QDateTime>
@@ -65,7 +37,9 @@ int gettimeofday(struct timeval * tp, struct timezone * tzp)
 #include <QtGlobal>
 #include <QToolTip>
 #include "plotter.h"
+#include "bandplan.h"
 #include "bookmarks.h"
+#include "dxc_spots.h"
 
 // Comment out to enable plotter debug messages
 //#define PLOTTER_DEBUG
@@ -86,6 +60,9 @@ int gettimeofday(struct timeval * tp, struct timezone * tzp)
 #define PLOTTER_FILTER_BOX_COLOR    0xFFA0A0A4
 // FIXME: Should cache the QColors also
 
+#define HOR_MARGIN 5
+#define VER_MARGIN 5
+
 int F2B(float f)
 {
     int b = (f >= 1.0 ? 255 : (f <= 0.0 ? 0 : (int)floor(f * 256.0)));
@@ -104,16 +81,6 @@ static inline bool out_of_range(float min, float max)
             max < min + 10.f);
 }
 
-/** Current time in milliseconds since Epoch */
-static inline quint64 time_ms(void)
-{
-    struct timeval  tval;
-
-    gettimeofday(&tval, NULL);
-
-    return 1e3 * tval.tv_sec + 1e-3 * tval.tv_usec;
-}
-
 #define STATUS_TIP \
     "Click, drag or scroll on spectrum to tune. " \
     "Drag and scroll X and Y axes for pan and zoom. " \
@@ -154,8 +121,10 @@ CPlotter::CPlotter(QWidget *parent) : QFrame(parent)
 
     m_FilterBoxEnabled = true;
     m_CenterLineEnabled = true;
+    m_BandPlanEnabled = true;
     m_BookmarksEnabled = true;
     m_InvertScrolling = false;
+    m_DXCSpotsEnabled = true;
 
     m_Span = 96000;
     m_SampleFreq = 96000;
@@ -177,6 +146,7 @@ CPlotter::CPlotter(QWidget *parent) : QFrame(parent)
     m_Percent2DScreen = 35;	//percent of screen used for 2D display
     m_VdivDelta = 30;
     m_HdivDelta = 70;
+    m_BandPlanHeight = 22;
 
     m_FreqDigits = 3;
 
@@ -223,17 +193,36 @@ void CPlotter::mouseMoveEvent(QMouseEvent* event)
             bool onTag = false;
             if(pt.y() < 15 * 10) // FIXME
             {
-                for(int i = 0; i < m_BookmarkTags.size() && !onTag; i++)
+                if(m_BookmarksEnabled || m_DXCSpotsEnabled)
                 {
-                    if (m_BookmarkTags[i].first.contains(event->pos()))
-                        onTag = true;
+                    for(int i = 0; i < m_Taglist.size() && !onTag; i++)
+                    {
+                        if (m_Taglist[i].first.contains(event->pos()))
+                            onTag = true;
+                    }
                 }
             }
             // if no mouse button monitor grab regions and change cursor icon
             if (onTag)
             {
                 setCursor(QCursor(Qt::PointingHandCursor));
-                m_CursorCaptured = BOOKMARK;
+                m_CursorCaptured = TAG;
+            }
+            else if (isPointCloseTo(pt.x(), m_YAxisWidth/2, m_YAxisWidth/2))
+            {
+                if (YAXIS != m_CursorCaptured)
+                    setCursor(QCursor(Qt::OpenHandCursor));
+                m_CursorCaptured = YAXIS;
+                if (m_TooltipsEnabled)
+                    QToolTip::hideText();
+            }
+            else if (isPointCloseTo(pt.y(), m_XAxisYCenter, m_CursorCaptureDelta+20))
+            {
+                if (XAXIS != m_CursorCaptured)
+                    setCursor(QCursor(Qt::OpenHandCursor));
+                m_CursorCaptured = XAXIS;
+                if (m_TooltipsEnabled)
+                    QToolTip::hideText();
             }
             else if (isPointCloseTo(pt.x(), m_DemodFreqX, m_CursorCaptureDelta))
             {
@@ -271,22 +260,6 @@ void CPlotter::mouseMoveEvent(QMouseEvent* event)
                                                .arg(m_DemodLowCutFreq),
                                        this);
             }
-            else if (isPointCloseTo(pt.x(), m_YAxisWidth/2, m_YAxisWidth/2))
-            {
-                if (YAXIS != m_CursorCaptured)
-                    setCursor(QCursor(Qt::OpenHandCursor));
-                m_CursorCaptured = YAXIS;
-                if (m_TooltipsEnabled)
-                    QToolTip::hideText();
-            }
-            else if (isPointCloseTo(pt.y(), m_XAxisYCenter, m_CursorCaptureDelta+5))
-            {
-                if (XAXIS != m_CursorCaptured)
-                    setCursor(QCursor(Qt::OpenHandCursor));
-                m_CursorCaptured = XAXIS;
-                if (m_TooltipsEnabled)
-                    QToolTip::hideText();
-            }
             else
             {	//if not near any grab boundaries
                 if (NOCAP != m_CursorCaptured)
@@ -295,10 +268,20 @@ void CPlotter::mouseMoveEvent(QMouseEvent* event)
                     m_CursorCaptured = NOCAP;
                 }
                 if (m_TooltipsEnabled)
-                    QToolTip::showText(event->globalPos(),
-                                       QString("F: %1 kHz")
-                                               .arg(freqFromX(pt.x())/1.e3f, 0, 'f', 3),
-                                       this);
+                {
+                    qint64 hoverFrequency = freqFromX(pt.x());
+                    QString toolTipText = QString("F: %1 kHz").arg(hoverFrequency/1.e3f, 0, 'f', 3);
+                    QFontMetrics metrics(m_Font);
+                    int bandTopY = m_OverlayPixmap.height() - metrics.height() - 2 * VER_MARGIN - m_BandPlanHeight;
+                    QList<BandInfo> hoverBands = BandPlan::Get().getBandsEncompassing(hoverFrequency);
+                    if(m_BandPlanEnabled && pt.y() > bandTopY && hoverBands.size() > 0)
+                    {
+                        toolTipText.append("\n");
+                        for (int i = 0; i < hoverBands.size(); i++)
+                            toolTipText.append("\n" + hoverBands[i].name);
+                    }
+                    QToolTip::showText(event->globalPos(), toolTipText, this);
+                }
             }
             m_GrabPosition = 0;
         }
@@ -698,13 +681,13 @@ void CPlotter::mousePressEvent(QMouseEvent * event)
                 resetHorizontalZoom();
             }
         }
-        else if (m_CursorCaptured == BOOKMARK)
+        else if (m_CursorCaptured == TAG)
         {
-            for (int i = 0; i < m_BookmarkTags.size(); i++)
+            for (int i = 0; i < m_Taglist.size(); i++)
             {
-                if (m_BookmarkTags[i].first.contains(event->pos()))
+                if (m_Taglist[i].first.contains(event->pos()))
                 {
-                    m_DemodCenterFreq = m_BookmarkTags[i].second;
+                    m_DemodCenterFreq = m_Taglist[i].second;
                     emit newDemodFreq(m_DemodCenterFreq, m_DemodCenterFreq - m_CenterFreq);
                     break;
                 }
@@ -786,7 +769,7 @@ void CPlotter::zoomOnXAxis(float level)
 void CPlotter::wheelEvent(QWheelEvent * event)
 {
     QPoint pt = event->pos();
-    int delta = m_InvertScrolling? -event->delta() : event->delta();
+    int delta = m_InvertScrolling? -event->angleDelta().y() : event->angleDelta().y();
     int numDegrees = delta / 8;
     int numSteps = numDegrees / 15;  /** FIXME: Only used for direction **/
 
@@ -922,7 +905,7 @@ void CPlotter::draw()
     // no need to draw if pixmap is invisible
     if (w != 0 && h != 0)
     {
-        quint64     tnow_ms = time_ms();
+        quint64     tnow_ms = QDateTime::currentMSecsSinceEpoch();
 
         // get scaled FFT data
         n = qMin(w, MAX_SCREENSIZE);
@@ -995,11 +978,9 @@ void CPlotter::draw()
 
         QPainter painter2(&m_2DPixmap);
 
-// workaround for "fixed" line drawing since Qt 5
-// see http://stackoverflow.com/questions/16990326
-#if QT_VERSION >= 0x050000
+        // workaround for "fixed" line drawing since Qt 5
+        // see http://stackoverflow.com/questions/16990326
         painter2.translate(0.5, 0.5);
-#endif
 
         // get new scaled fft data
         getScreenIntegerFFTData(h, qMin(w, MAX_SCREENSIZE),
@@ -1282,8 +1263,7 @@ void CPlotter::drawOverlay()
     painter.setBrush(Qt::SolidPattern);
     painter.fillRect(0, 0, w, h, QColor(PLOTTER_BGD_COLOR));
 
-#define HOR_MARGIN 5
-#define VER_MARGIN 5
+    QList<BookmarkInfo> tags;
 
     // X and Y axis areas
     m_YAxisWidth = metrics.boundingRect("XXXX").width() + 2 * HOR_MARGIN;
@@ -1292,26 +1272,43 @@ void CPlotter::drawOverlay()
     int xAxisTop = h - xAxisHeight;
     int fLabelTop = xAxisTop + VER_MARGIN;
 
-    if (m_BookmarksEnabled)
+    if (m_BookmarksEnabled || m_DXCSpotsEnabled)
     {
-        m_BookmarkTags.clear();
+        m_Taglist.clear();
         static const QFontMetrics fm(painter.font());
         static const int fontHeight = fm.ascent() + 1;
         static const int slant = 5;
         static const int levelHeight = fontHeight + 5;
-        const int nLevels = h / (levelHeight + slant);
-        QList<BookmarkInfo> bookmarks = Bookmarks::Get().getBookmarksInRange(m_CenterFreq + m_FftCenter - m_Span / 2,
-                                                                             m_CenterFreq + m_FftCenter + m_Span / 2);
-        QVector<int> tagEnd(nLevels + 1);
-        for (int i = 0; i < bookmarks.size(); i++)
+        static const int nLevels = h / (levelHeight + slant);
+        if (m_BookmarksEnabled)
         {
-            x = xFromFreq(bookmarks[i].frequency);
-
-#if defined(_WIN16) || defined(_WIN32) || defined(_WIN64)
-            int nameWidth = fm.width(bookmarks[i].name);
-#else
-            int nameWidth = fm.boundingRect(bookmarks[i].name).width();
-#endif
+            tags = Bookmarks::Get().getBookmarksInRange(m_CenterFreq + m_FftCenter - m_Span / 2,
+                                                        m_CenterFreq + m_FftCenter + m_Span / 2);
+        }
+        else
+        {
+            tags.clear();
+        }
+        if (m_DXCSpotsEnabled)
+        {
+            QList<DXCSpotInfo> dxcspots = DXCSpots::Get().getDXCSpotsInRange(m_CenterFreq + m_FftCenter - m_Span / 2,
+                                                                             m_CenterFreq + m_FftCenter + m_Span / 2);
+            QListIterator<DXCSpotInfo> iter(dxcspots);
+            while(iter.hasNext())
+            {
+                BookmarkInfo tempDXCSpot;
+                DXCSpotInfo IterDXCSpot = iter.next();
+                tempDXCSpot.name = IterDXCSpot.name;
+                tempDXCSpot.frequency = IterDXCSpot.frequency;
+                tags.append(tempDXCSpot);
+            }
+            std::stable_sort(tags.begin(),tags.end());
+        }
+        QVector<int> tagEnd(nLevels + 1);
+        for (int i = 0; i < tags.size(); i++)
+        {
+            x = xFromFreq(tags[i].frequency);
+            int nameWidth = fm.boundingRect(tags[i].name).width();
 
             int level = 0;
             while(level < nLevels && tagEnd[level] > x)
@@ -1330,9 +1327,9 @@ void CPlotter::drawOverlay()
             const auto levelNHeightBottom = levelNHeight + fontHeight;
             const auto levelNHeightBottomSlant = levelNHeightBottom + slant;
 
-            m_BookmarkTags.append(qMakePair<QRect, qint64>(QRect(x, levelNHeight, nameWidth + slant, fontHeight), bookmarks[i].frequency));
+            m_Taglist.append(qMakePair<QRect, qint64>(QRect(x, levelNHeight, nameWidth + slant, fontHeight), tags[i].frequency));
 
-            QColor color = QColor(bookmarks[i].GetColor());
+            QColor color = QColor(tags[i].GetColor());
             color.setAlpha(0x60);
             // Vertical line
             painter.setPen(QPen(color, 1, Qt::DashLine));
@@ -1351,7 +1348,31 @@ void CPlotter::drawOverlay()
             painter.setPen(QPen(color, 2, Qt::SolidLine));
             painter.drawText(x + slant, levelNHeight, nameWidth,
                              fontHeight, Qt::AlignVCenter | Qt::AlignHCenter,
-                             bookmarks[i].name);
+                             tags[i].name);
+        }
+    }
+
+    if (m_BandPlanEnabled)
+    {
+        QList<BandInfo> bands = BandPlan::Get().getBandsInRange(m_CenterFreq + m_FftCenter - m_Span / 2,
+                                                                m_CenterFreq + m_FftCenter + m_Span / 2);
+
+        for (int i = 0; i < bands.size(); i++)
+        {
+            int band_left = xFromFreq(bands[i].minFrequency);
+            int band_right = xFromFreq(bands[i].maxFrequency);
+            int band_width = band_right - band_left;
+            rect.setRect(band_left, xAxisTop - m_BandPlanHeight, band_width, m_BandPlanHeight);
+            painter.fillRect(rect, bands[i].color);
+            QString band_label = bands[i].name + " (" + bands[i].modulation + ")";
+            int textWidth = metrics.boundingRect(band_label).width();
+            if (band_left < w && band_width > textWidth + 20)
+            {
+                painter.setOpacity(1.0);
+                rect.setRect(band_left, xAxisTop - m_BandPlanHeight, band_width, metrics.height());
+                painter.setPen(QColor(PLOTTER_TEXT_COLOR));
+                painter.drawText(rect, Qt::AlignCenter, band_label);
+            }
         }
     }
 
@@ -1539,7 +1560,7 @@ int CPlotter::xFromFreq(qint64 freq)
     return x;
 }
 
-// Convert from frequency to screen coordinate
+// Convert from screen coordinate to frequency
 qint64 CPlotter::freqFromX(int x)
 {
     qint64 w = width();
@@ -1685,6 +1706,12 @@ void CPlotter::setPeakDetection(bool enabled, float c)
         m_PeakDetection = c;
 }
 
+void CPlotter::toggleBandPlan(bool state)
+{
+    m_BandPlanEnabled = state;
+    updateOverlay();
+}
+
 void CPlotter::calcDivSize (qint64 low, qint64 high, int divswanted, qint64 &adjlow, qint64 &step, int& divs)
 {
 #ifdef PLOTTER_DEBUG
