@@ -1,7 +1,7 @@
 /* -*- c++ -*- */
 /*
  * Gqrx SDR: Software defined radio receiver powered by GNU Radio and Qt
- *           http://gqrx.dk/
+ *           https://gqrx.dk/
  *
  * Copyright 2011-2016 Alexandru Csete OZ9AEC.
  *
@@ -50,6 +50,7 @@ nbrx::nbrx(float quad_rate, float audio_rate)
     demod_ssb = gr::blocks::complex_to_real::make(1);
     demod_fm = make_rx_demod_fm(PREF_QUAD_RATE, 5000.0, 75.0e-6);
     demod_am = make_rx_demod_am(PREF_QUAD_RATE, true);
+    demod_amsync = make_rx_demod_amsync(PREF_QUAD_RATE, true, 0.001);
 
     audio_rr0.reset();
     audio_rr1.reset();
@@ -256,6 +257,11 @@ void nbrx::set_demod(int rx_demod)
         demod = demod_am;
         break;
 
+    case NBRX_DEMOD_AMSYNC:
+        d_demod = NBRX_DEMOD_AMSYNC;
+        demod = demod_amsync;
+        break;
+
     case NBRX_DEMOD_FM:
     default:
         d_demod = NBRX_DEMOD_FM;
@@ -311,3 +317,13 @@ void nbrx::set_am_dcr(bool enabled)
 {
     demod_am->set_dcr(enabled);
 }
+
+void nbrx::set_amsync_dcr(bool enabled)
+{
+    demod_amsync->set_dcr(enabled);
+}
+
+void nbrx::set_amsync_pll_bw(float pll_bw)
+{
+    demod_amsync->set_pll_bw(pll_bw);
+}
